#include <jni.h>
#include <string>
#include <android/native_window_jni.h>
#include "ivr4pengine.h"
#include <EGL/egl.h>
#include <EGL/eglext.h>
#include <GLES2/gl2.h>
#include "Context.h"


int  iWidth = 1920;
int  iHeight = 1080;

extern ANativeWindow * g_pNativeWindow;
extern EGLDisplay      g_mEglDisplay;
extern EGLSurface      g_mEglSurface;
extern EGLContext      g_mEglContext;


bool    InitContext(bool bUseFrontBuffer, bool bUseMutable)
{
    if(bUseFrontBuffer)
    {
        EGLContext  eTexUpdateContext = (EGLContext)V4_ResetGLTexUpdatePT();
        if(eTexUpdateContext == EGL_NO_CONTEXT)
            return false;

        g_mEglDisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
        if (EGL_FALSE == eglInitialize(g_mEglDisplay, 0, 0))
            return false;

        EGLConfig  mEglConfigMain;
        EGLint numConfigs = 0;

        const EGLint attribs[] =
                {
                        EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
                        EGL_SURFACE_TYPE, bUseMutable?(EGL_WINDOW_BIT|EGL_MUTABLE_RENDER_BUFFER_BIT_KHR):EGL_WINDOW_BIT,
                        EGL_BLUE_SIZE,    8,
                        EGL_GREEN_SIZE,   8,
                        EGL_RED_SIZE,     8,
                        EGL_ALPHA_SIZE,   8,
                        EGL_DEPTH_SIZE,	0,
                        EGL_STENCIL_SIZE,	8,
                        EGL_SAMPLES,		0,
                        EGL_NONE
                };

        eglChooseConfig(g_mEglDisplay, attribs, &mEglConfigMain, 1, &numConfigs);

        static const EGLint surf_attribute_list[] = {
                EGL_RENDER_BUFFER, EGL_SINGLE_BUFFER,
                EGL_NONE
        };

        g_mEglSurface = eglCreateWindowSurface(g_mEglDisplay, mEglConfigMain, g_pNativeWindow, surf_attribute_list);
        if (g_mEglSurface == EGL_NO_SURFACE)
            return false;

        eglSurfaceAttrib(g_mEglDisplay,g_mEglSurface,EGL_SWAP_BEHAVIOR,EGL_BUFFER_PRESERVED);

        EGLint attribList2[] = { EGL_CONTEXT_CLIENT_VERSION, 2,EGL_CONTEXT_PRIORITY_LEVEL_IMG,EGL_CONTEXT_PRIORITY_HIGH_IMG, EGL_NONE };

        g_mEglContext = eglCreateContext(g_mEglDisplay, mEglConfigMain, eTexUpdateContext, attribList2);
        if (g_mEglContext == EGL_NO_CONTEXT)
            return false;

        if (EGL_FALSE == eglMakeCurrent(g_mEglDisplay, g_mEglSurface, g_mEglSurface, g_mEglContext))
            return false;

        eglQuerySurface(g_mEglDisplay, g_mEglSurface, EGL_WIDTH, &iWidth);
        eglQuerySurface(g_mEglDisplay, g_mEglSurface, EGL_HEIGHT, &iHeight);

        glBindFramebuffer(GL_FRAMEBUFFER, 0);
        glViewport(0,0,iWidth,iHeight);
        glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
        glClearStencil(0);
        glClear(GL_COLOR_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);

        V4_ResizeMyGL(iWidth,iHeight);
        V4_ResetGLRes();
        return true;
    }
    else
    {
        g_mEglDisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
        if (EGL_FALSE == eglInitialize(g_mEglDisplay, 0, 0))
            return false;

        EGLConfig  mEglConfigMain;
        EGLint numConfigs = 0;

        const EGLint attribs[] =
                {
                        EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
                        EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
                        EGL_BLUE_SIZE,    8,
                        EGL_GREEN_SIZE,   8,
                        EGL_RED_SIZE,     8,
                        EGL_ALPHA_SIZE,   8,
                        EGL_DEPTH_SIZE,	0,
                        EGL_STENCIL_SIZE,	8,
                        EGL_SAMPLES,		0,
                        EGL_NONE
                };

        eglChooseConfig(g_mEglDisplay, attribs, &mEglConfigMain, 1, &numConfigs);

        g_mEglSurface = eglCreateWindowSurface(g_mEglDisplay, mEglConfigMain, g_pNativeWindow, NULL);
        if (g_mEglSurface == EGL_NO_SURFACE)
            return false;

        eglSurfaceAttrib(g_mEglDisplay,g_mEglSurface,EGL_SWAP_BEHAVIOR,EGL_BUFFER_PRESERVED);

        EGLint attribList2[] = { EGL_CONTEXT_CLIENT_VERSION, 2,EGL_CONTEXT_PRIORITY_LEVEL_IMG,EGL_CONTEXT_PRIORITY_HIGH_IMG, EGL_NONE };

        g_mEglContext = eglCreateContext(g_mEglDisplay, mEglConfigMain, NULL, attribList2);
        if (g_mEglContext == EGL_NO_CONTEXT)
            return false;

        if (EGL_FALSE == eglMakeCurrent(g_mEglDisplay, g_mEglSurface, g_mEglSurface, g_mEglContext))
            return false;

        eglQuerySurface(g_mEglDisplay, g_mEglSurface, EGL_WIDTH, &iWidth);
        eglQuerySurface(g_mEglDisplay, g_mEglSurface, EGL_HEIGHT, &iHeight);

        glBindFramebuffer(GL_FRAMEBUFFER, 0);
        glViewport(0,0,iWidth,iHeight);
        glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
        glClearStencil(0);
        glClear(GL_COLOR_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);

        V4_ResizeMyGL(iWidth,iHeight);
        V4_ResetGLRes();
        return true;
    }
}

void    UninitContext(bool bUseFrontBuffer)
{
    V4_ReleaseGLRes();

    if(g_mEglDisplay != NULL && g_mEglContext != EGL_NO_CONTEXT)
    {
        eglMakeCurrent(g_mEglDisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
        eglDestroyContext(g_mEglDisplay,g_mEglContext);
        g_mEglContext = EGL_NO_CONTEXT;
    }

    if(g_mEglDisplay != NULL && g_mEglSurface != EGL_NO_SURFACE)
    {
        eglDestroySurface(g_mEglDisplay,g_mEglSurface);
        g_mEglSurface = EGL_NO_CONTEXT;
    }

    if(g_mEglDisplay != NULL)
    {
        eglTerminate(g_mEglDisplay);
        g_mEglDisplay = NULL;
    }
    if(bUseFrontBuffer)
        V4_ReleaseGLTexUpdatePT();
}








