#include <jni.h>
#include <string>
#include <android/log.h>
#include <android/window.h>
#include <android/native_window_jni.h>
#include <android/input.h>
#include <android/native_activity.h>
#include "android_native_app_glue.h"
#include "ivr4pengine.h"
#include <EGL/egl.h>
#include <EGL/eglext.h>
#include <unistd.h>
#include <GLES2/gl2.h>

#define LOG_TAG "VR4P"

#define ALOGE(...) __android_log_print( ANDROID_LOG_ERROR, LOG_TAG, __VA_ARGS__ )
#if DEBUG
#define ALOGV(...) __android_log_print( ANDROID_LOG_VERBOSE, LOG_TAG, __VA_ARGS__ )
#else
#define ALOGV(...)
#endif
#include <vector>

using namespace std;

ANativeWindow * g_pNativeWindow = NULL;
EGLDisplay      g_mEglDisplay = NULL;
EGLSurface      g_mEglSurface = EGL_NO_SURFACE;
EGLContext      g_mEglContext = EGL_NO_CONTEXT;
vector<bool>    g_vAllResumedBuffer;
int32_t          g_iSleepTime = 200;



double GetTimeInSeconds()
{
    struct timespec now;
    clock_gettime( CLOCK_MONOTONIC, &now );
    return ( now.tv_sec * 1e9 + now.tv_nsec ) * 0.000000001;
}

void app_handle_cmd( struct android_app * app, int32_t cmd )
{
    ALOGE("app_handle_cmd=%d",cmd);

    switch ( cmd )
    {
        case APP_CMD_START:
            break;
        case APP_CMD_RESUME:
        {
            if(g_vAllResumedBuffer.size() == 0 || !g_vAllResumedBuffer.back())
                g_vAllResumedBuffer.push_back(true);
            break;
        }
        case APP_CMD_PAUSE:
        {
            if(g_vAllResumedBuffer.size() == 0 || g_vAllResumedBuffer.back())
                g_vAllResumedBuffer.push_back(false);
            break;
        }
        case APP_CMD_STOP:
            break;
        case APP_CMD_DESTROY:
        {
            g_pNativeWindow = NULL;
            break;
        }
        case APP_CMD_INIT_WINDOW:
        {
            g_pNativeWindow = app->window;
            break;
        }
        case APP_CMD_TERM_WINDOW:
        {
            g_pNativeWindow = NULL;
            break;
        }
        default:break;
    }
}

int32_t app_handle_input( struct android_app * app, AInputEvent * event )
{
    if(event != NULL)
    {
        int32_t iCode = AKeyEvent_getKeyCode(event);
        int32_t itype = AInputEvent_getType( event );
        int32_t iaction = AKeyEvent_getAction( event );
        ALOGE("app_handle_input iCode=%d itype=%d iaction=%d",iCode,itype,iaction);
    }
    return 0;
}

#include "Context.h"

void  HandleVrModeChanges()
{
    while(g_vAllResumedBuffer.size() >= 2)
    {
        if(g_vAllResumedBuffer[0] && !g_vAllResumedBuffer[1])
        {
            g_vAllResumedBuffer.erase(g_vAllResumedBuffer.begin(),g_vAllResumedBuffer.begin()+2);
            continue;
        }
        else
            break;
    }

    if(g_vAllResumedBuffer.size() == 0)
        g_vAllResumedBuffer.push_back(false);

    bool bResumed = g_vAllResumedBuffer[0];

    if(g_vAllResumedBuffer.size() > 1)
        g_vAllResumedBuffer.erase(g_vAllResumedBuffer.begin());

    if(g_pNativeWindow != NULL && bResumed)
    {
        if(g_mEglDisplay == NULL)
        {
            if(V4_IsFrontBuffer())
            {
                if(!InitContext(true,true))
                {
                    UninitContext(true);
                    if(!InitContext(true,false))
                    {
                        UninitContext(true);
                        V4_UseFrontBuffer(false);
                        InitContext(false,false);
                    }
                }
            }
            else
                InitContext(false,false);
        }
    }
    else
    {
        if(g_mEglDisplay != NULL)
            UninitContext(V4_IsFrontBuffer());
    }
}


pthread_t           g_android_Render = (pthread_t)NULL;
volatile bool	    g_bGameEndDV = false;
volatile bool	    g_bVRSensorThread = true;

void * android_Render(void *)
{
    double  dTimeForLooping = GetTimeInSeconds();
    double  fFramwWaitTime = 1.0 / 60.0;
    double  fCompareTime = double(g_iSleepTime) / 2000000.0;

    if(g_vAllResumedBuffer.size() == 0)
        g_vAllResumedBuffer.push_back(false);
    g_bVRSensorThread = false;

    while (!g_bGameEndDV)
    {
        if(!V4_IsFrontBuffer() || !V4_CfgGetGlassMode())
        {
            HandleVrModeChanges();
            double thisTime = GetTimeInSeconds();

            if(thisTime - dTimeForLooping < fFramwWaitTime - fCompareTime)
            {
                usleep((useconds_t) g_iSleepTime);
            }
            else
            {
                while(thisTime - dTimeForLooping >= fFramwWaitTime - fCompareTime * 2.0)
                    dTimeForLooping += fFramwWaitTime;

                if(g_mEglDisplay != NULL)
                {
                    eglSwapBuffers(g_mEglDisplay,g_mEglSurface);
                    glBindFramebuffer(GL_FRAMEBUFFER, 0);
                    V4_PreRender();

                    if(V4_CfgGetGlassMode())
                    {
                        V4_RenderScene(g_iRenderScene_VRLeft);
                        V4_RenderScene(g_iRenderScene_VRRight);
                    }
                    else
                    {
                        V4_RenderScene(g_iRenderScene_NoGlass);
                    }
                    V4_AftRender();
                }
            }
        }
        else
        {
            HandleVrModeChanges();

            if(!V4_NeedRenderVR())
            {
                V4_CfgGetUndistortParam1();
                usleep((useconds_t) g_iSleepTime);
                V4_CfgGetUndistortParam2();
            }
            else
            {
                if(g_mEglDisplay != NULL)
                {
                    V4_CfgGetCenterOffsetH();
                    eglSwapBuffers(g_mEglDisplay,g_mEglSurface);
                    glBindFramebuffer(GL_FRAMEBUFFER, 0);
                    V4_CfgGetCenterOffsetV();
                    V4_PreRender();
                    V4_RenderScene(g_iRenderScene_VRLeft);
                    V4_RenderScene(g_iRenderScene_VRRight);
                    V4_AftRender();
                }
            }
        }
    }

    g_vAllResumedBuffer.resize(1);
    g_vAllResumedBuffer.back() = false;
    HandleVrModeChanges();

    g_bVRSensorThread = true;
    pthread_exit((void *)0);
}


JavaVM *	g_Vm = NULL;
JNIEnv *	g_Env = NULL;

#include <sys/prctl.h>

void android_main( struct android_app * app )
{
    ANativeActivity_setWindowFlags( app->activity, AWINDOW_FLAG_KEEP_SCREEN_ON, 0 );
    g_Vm = app->activity->vm;
    (*g_Vm).AttachCurrentThread(&g_Env, NULL );

    prctl( PR_SET_NAME, (long)"VR4PNATIVE::Main", 0, 0, 0 );

    app->userData = NULL;
    app->onAppCmd = app_handle_cmd;
    app->onInputEvent = app_handle_input;

    pthread_create(&g_android_Render, NULL, android_Render, NULL);

    while ( app->destroyRequested == 0 )
    {
        for (;;)
        {
            int events;
            struct android_poll_source *source;
            if (ALooper_pollAll(-1, NULL, &events, (void **) &source) < 0)
                break;
            if (source != NULL)
                source->process(app, source);
        }
    }

    g_bGameEndDV = true;
    while(!g_bVRSensorThread)
        pthread_join(g_android_Render,NULL);

    (*g_Vm).DetachCurrentThread();
}





